@@ -198,11 +198,23 @@ typedef struct {
 #define EM8300_LOGNAME "em8300"
 extern int major;
 
+#include <linux/version.h>
+#include <linux/types.h> /* ulong, uint32_t */
+#include <linux/i2c.h> /* struct i2c_adapter */
+#include <linux/i2c-algo-bit.h> /* struct i2c_algo_bit_data */
+#include <linux/time.h> /* struct timeval */
+#include <linux/wait.h> /* wait_queue_head_t */
+#include <linux/list.h> /* struct list_head */
+
 #if defined(CONFIG_SND) || defined(CONFIG_SND_MODULE)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
+#define snd_card_t struct snd_card
+#else
 #include <sound/driver.h>
 #include <sound/core.h>
 #include <sound/pcm.h>
 #endif
+#endif
 
 struct dicom_s {
 	int luma;
@@ -337,6 +349,7 @@ struct em8300_s
 	int video_mode;
 	int video_playmode;
 	int aspect_ratio;
+	int zoom;
 	uint32_t video_pts;
 	uint32_t video_lastpts;
 	int video_ptsvalid,video_offset,video_count;
@@ -386,10 +399,19 @@ struct em8300_s
 	struct list_head  memory;
 #endif	
 
+	/* Checksum for the on-board eeprom */
+	u8 *eeprom_checksum;
+
 	/* To support different options for different cards */
 	unsigned int card_nr;
 };
 
+#if defined(CONFIG_SND) || defined(CONFIG_SND_MODULE)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
+#undef snd_card_t
+#endif
+#endif
+
 #define TIMEDIFF(a,b) a.tv_usec - b.tv_usec + \
 	    1000000 * (a.tv_sec - b.tv_sec)
 
@@ -399,7 +421,8 @@ struct em8300_s
 */
 
 /* em8300_i2c.c */
-int em8300_i2c_init(struct em8300_s *em);
+int em8300_i2c_init1(struct em8300_s *em);
+int em8300_i2c_init2(struct em8300_s *em);
 void em8300_i2c_exit(struct em8300_s *em);
 void em8300_clockgen_write(struct em8300_s *em, int abyte);
 
