@@ -19,10 +19,14 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
+#include <linux/version.h>
 #include <linux/module.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,10)
+#include <linux/moduleparam.h>
+#endif
 #include <linux/init.h>
 #include <linux/delay.h>
 #include <linux/errno.h>
@@ -37,11 +41,9 @@
 #include <asm/pgtable.h>
 #include <asm/page.h>
 #include <linux/sched.h>
-#include <asm/segment.h>
 #include <linux/types.h>
 
 #include <linux/videodev.h>
-#include <linux/version.h>
 #include <asm/uaccess.h>
 
 #include <linux/i2c.h>
@@ -54,28 +56,51 @@
 
 MODULE_SUPPORTED_DEVICE("bt865");
 MODULE_LICENSE("GPL");
+#ifdef MODULE_VERSION
+MODULE_VERSION(EM8300_VERSION);
+#endif
 
 EXPORT_NO_SYMBOLS;
 
 static int color_bars[EM8300_MAX] = { [ 0 ... EM8300_MAX-1 ] = 0 };
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,10)
 MODULE_PARM(color_bars, "1-" __MODULE_STRING(EM8300_MAX) "i");
+#else
+module_param_array(color_bars, bool, NULL, 0444);
+#endif
 MODULE_PARM_DESC(color_bars, "If you set this to 1 a set of color bars will be displayed on your screen (used for testing if the chip is working). Defaults to 0.");
 
-static int rgb_mode[EM8300_MAX] = { [ 0 ... EM8300_MAX-1 ] = 0 };
-MODULE_PARM(rgb_mode, "1-" __MODULE_STRING(EM8300_MAX) "i");
-MODULE_PARM_DESC(rgb_mode, "If you set this to 1, RGB output is enabled. You will need to hack the DXR3 hardware. Defaults to 0.");
+typedef enum {
+	MODE_COMPOSITE_SVIDEO,
+	MODE_RGB,
+	MODE_MAX
+} output_mode_t;
 
-#define i2c_is_isa_client(clientptr) \
-		((clientptr)->adapter->algo->id == I2C_ALGO_ISA)
-#define i2c_is_isa_adapter(adapptr) \
-		((adapptr)->algo->id == I2C_ALGO_ISA)
+struct output_conf_s {
+};
 
+#include "encoder_output_mode.h"
+
+static output_mode_t output_mode_nr[EM8300_MAX] = { [ 0 ... EM8300_MAX-1 ] = MODE_COMPOSITE_SVIDEO };
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,10)
+static char *output_mode[EM8300_MAX] = { [ 0 ... EM8300_MAX-1 ] = NULL };
+MODULE_PARM(output_mode, "1-" __MODULE_STRING(EM8300_MAX) "s");
+#else
+module_param_array_named(output_mode, output_mode_nr, output_mode_t, NULL, 0444);
+#endif
+MODULE_PARM_DESC(output_mode, "Specifies the output mode to use for the BT865 video encoder. See the README-modoptions file for the list of mode names to use. Default is SVideo + composite (\"comp+svideo\").");
 
 static int bt865_attach_adapter(struct i2c_adapter *adapter);
 int bt865_detach_client(struct i2c_client *client);
 int bt865_command(struct i2c_client *client, unsigned int cmd, void *arg);
 static int bt865_setup(struct i2c_client *client);
 
+static const mode_info_t mode_info[] = {
+	[ MODE_COMPOSITE_SVIDEO ] =		{ "comp+svideo" , { } },
+	[ MODE_RGB ] =				{ "rgb"         , { } },
+};
+
 struct bt865_data_s {
 	int chiptype;
 	int mode;
@@ -89,12 +114,18 @@ struct bt865_data_s {
 
 /* This is the driver that will be inserted */
 static struct i2c_driver bt865_driver = {
-#if defined(EM8300_I2C_FORCE_NEW_API) || (LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,54) && !defined(EM8300_I2C_FORCE_OLD_API))
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
+	.driver = {
+		.name =		"bt865",
+	},
+#else
+#if defined(EM8300_I2C_FORCE_NEW_API) || LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,54)
 	.owner =		THIS_MODULE,
 #endif
-	.name =			"BT865 video encoder driver",
-	.id =			I2C_DRIVERID_BT865,
+	.name =			"bt865",
 	.flags =		I2C_DF_NOTIFY,
+#endif
+	.id =			I2C_DRIVERID_BT865,
 	.attach_adapter =	&bt865_attach_adapter,
 	.detach_client =	&bt865_detach_client,
 	.command =		&bt865_command
@@ -846,7 +877,7 @@ static int bt865_setup(struct i2c_client *client)
 	}
 
 	data->bars = color_bars[em->card_nr];
-	data->rgbmode = rgb_mode[em->card_nr];
+	data->rgbmode = output_mode_nr[em->card_nr] == MODE_RGB;
 	data->enableoutput = 0;
 
 	if (EM8300_VIDEOMODE_DEFAULT == EM8300_VIDEOMODE_PAL) {
@@ -871,10 +902,6 @@ static int bt865_detect(struct i2c_adapter *adapter, int address)
 	struct i2c_client *new_client;
 	int err, chk = 0;
 
-	if (i2c_is_isa_adapter(adapter)) {
-		return 0;
-	}
-
 	chk = i2c_check_functionality(adapter,
 			I2C_FUNC_SMBUS_READ_BYTE | I2C_FUNC_SMBUS_WRITE_BYTE_DATA);
 
@@ -977,6 +1004,20 @@ int bt865_command(struct i2c_client *client, unsigned int cmd, void *arg)
 
 int __init bt865_init(void)
 {
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,10)
+	int i;
+	for (i=0; i < EM8300_MAX; i++)
+		if ((output_mode[i]) && (output_mode[i][0])) {
+			int j;
+			for (j=0; j < MODE_MAX; j++)
+				if (strcmp(output_mode[i], mode_info[j].name) == 0) {
+					output_mode_nr[i] = j;
+					break;
+				}
+			if (j == MODE_MAX)
+				printk(KERN_WARNING "bt865: Unknown output mode: %s\n", output_mode[i]);
+		}
+#endif /* ! CONFIG_MODULEPARAM */
 	//request_module("i2c-algo-bit");
 	return i2c_add_driver(&bt865_driver);
 }
